
# q-expansion of newform 168.4.a.h, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 168
weight = 4
poly_data = [-44, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [3, 0], [7, -1], [7, 0], [9, 3], [24, -2], [17, 9], [-8, 4], [55, -7], [106, -4], [-68, 4], [-12, 26], [347, -9], [-292, -8], [-158, -2], [280, 6], [-246, 26], [-302, -28], [-510, 10], [-855, -25], [-656, -6], [-278, -42], [-132, 32], [35, 95], [-68, -10], [-375, -51], [600, 32], [921, -77], [-890, 16], [-186, -20], [-1582, 30], [2056, -60], [-296, 18], [2220, -8], [1928, -10], [-368, 56], [-1218, 24], [910, 222], [-174, -202], [1285, -23], [703, -139], [284, 274], [-3403, 35], [-420, -234], [-240, 62], [1892, 20], [-1576, -204], [396, -236], [-2390, 50], [1940, 170], [184, 82], [-5129, -63], [-1052, 446], [-1702, 298], [-2685, -265], [207, -239], [1743, -465], [-880, -448], [-1712, -114], [-1212, -394], [5384, -116], [3945, 365], [-308, -272], [-582, -418], [-646, 336], [4464, 262], [-1072, -764], [3902, 340], [5727, 109], [-2974, 340], [-1431, 65], [2265, 119], [10520, 152], [9326, -224], [-2568, -724], [156, -220], [5786, -316], [-2954, -848], [-9636, 6], [8100, -394], [-4526, -662], [-8432, -386], [9165, 379], [-4730, -436], [-2016, -152], [-5705, 85], [10274, 352], [-2294, 400], [657, -867], [86, -310], [12270, -290], [1718, 410], [5116, -804], [7527, 373], [-11864, -252], [2336, -312], [-9417, 343], [-10299, 29], [-1012, -264], [-2528, 470], [2578, 1018], [11804, 650], [-13426, 254], [-16128, 58], [-13666, -426], [24822, 108], [-6226, 102], [-1527, -911], [-7291, 1115], [-19682, 12], [14656, -64], [3838, 1344], [3996, -690], [16416, 420], [-12454, 862], [1160, 1738], [6592, -852], [15246, 1082], [-2186, -1568], [2509, 247], [12278, -728], [28768, -46], [14085, 465], [-5085, -615], [5480, -1612], [-4342, 892], [5878, 1064], [-5208, -216], [7616, 1992], [11560, 638], [-13898, 1014], [-15335, 1143], [-5896, 2328], [-10010, -888], [-28389, -793], [4942, -140], [4085, 2081], [7960, -92], [12261, 89], [26298, -448], [-2688, -1532], [-5100, 1002], [-7354, 1874], [14913, -1605], [-13036, 1362], [-12618, -942], [-2630, -284], [2787, 1087], [-22172, 1904], [15355, 1933], [-13594, 944], [16301, -1459], [-7064, -956], [8850, 38], [34412, -1368], [-5083, 1475], [2620, -2996], [15743, 475], [-24378, 660], [-441, -1161], [31193, 1771], [-15262, 2056], [3690, -2338], [31556, 1792], [20556, -170], [-17364, -484], [-31452, 1340], [-48218, 104]]
